"""
Support for Qwikswitch relays.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/switch.qwikswitch/
"""
from homeassistant.components.qwikswitch import (
    QSToggleEntity, DOMAIN as QWIKSWITCH)
from homeassistant.components.switch import SwitchDevice

DEPENDENCIES = [QWIKSWITCH]


async def async_setup_platform(hass, _, add_devices, discovery_info=None):
    """Add switches from the main Qwikswitch component."""
    if discovery_info is None:
        return

    qsusb = hass.data[QWIKSWITCH]
    devs = [QSSwitch(qsid, qsusb) for qsid in discovery_info[QWIKSWITCH]]
    add_devices(devs)


class QSSwitch(QSToggleEntity, SwitchDevice):
    """Switch based on a Qwikswitch relay module."""



"""
Get your own public IP address or that of any host.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.dnsip/
"""
import asyncio
import logging
from datetime import timedelta

import voluptuous as vol

from homeassistant.const import STATE_UNKNOWN
from homeassistant.components.sensor import PLATFORM_SCHEMA
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.entity import Entity

REQUIREMENTS = ['aiodns==1.1.1']

_LOGGER = logging.getLogger(__name__)

CONF_HOSTNAME = 'hostname'
CONF_RESOLVER = 'resolver'
CONF_RESOLVER_IPV6 = 'resolver_ipv6'
CONF_IPV6 = 'ipv6'

DEFAULT_HOSTNAME = 'myip.opendns.com'
DEFAULT_RESOLVER = '208.67.222.222'
DEFAULT_RESOLVER_IPV6 = '2620:0:ccc::2'
DEFAULT_IPV6 = False

SCAN_INTERVAL = timedelta(seconds=120)

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Optional(CONF_HOSTNAME, default=DEFAULT_HOSTNAME): cv.string,
    vol.Optional(CONF_RESOLVER, default=DEFAULT_RESOLVER): cv.string,
    vol.Optional(CONF_RESOLVER_IPV6, default=DEFAULT_RESOLVER_IPV6): cv.string,
    vol.Optional(CONF_IPV6, default=DEFAULT_IPV6): cv.boolean,
})


@asyncio.coroutine
def async_setup_platform(hass, config, async_add_devices, discovery_info=None):
    """Set up the DNS IP sensor."""
    hostname = config.get(CONF_HOSTNAME)
    ipv6 = config.get(CONF_IPV6)
    if ipv6:
        resolver = config.get(CONF_RESOLVER_IPV6)
    else:
        resolver = config.get(CONF_RESOLVER)

    async_add_devices([WanIpSensor(
        hass, hostname, resolver, ipv6)], True)


class WanIpSensor(Entity):
    """Implementation of a DNS IP sensor."""

    def __init__(self, hass, hostname, resolver, ipv6):
        """Initialize the sensor."""
        import aiodns
        self.hass = hass
        self._name = hostname
        self.resolver = aiodns.DNSResolver(loop=self.hass.loop)
        self.resolver.nameservers = [resolver]
        self.querytype = 'AAAA' if ipv6 else 'A'
        self._state = STATE_UNKNOWN

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the current DNS IP address for hostname."""
        return self._state

    @asyncio.coroutine
    def async_update(self):
        """Get the current DNS IP address for hostname."""
        response = yield from self.resolver.query(self._name, self.querytype)
        if response:
            self._state = response[0].host
        else:
            self._state = STATE_UNKNOWN



"""
This platform provides binary sensors for key RainMachine data.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/binary_sensor.rainmachine/
"""
import logging

from homeassistant.components.binary_sensor import BinarySensorDevice
from homeassistant.components.rainmachine import (
    BINARY_SENSORS, DATA_RAINMACHINE, SENSOR_UPDATE_TOPIC, TYPE_FREEZE,
    TYPE_FREEZE_PROTECTION, TYPE_HOT_DAYS, TYPE_HOURLY, TYPE_MONTH,
    TYPE_RAINDELAY, TYPE_RAINSENSOR, TYPE_WEEKDAY, RainMachineEntity)
from homeassistant.const import CONF_MONITORED_CONDITIONS
from homeassistant.core import callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect

DEPENDENCIES = ['rainmachine']

_LOGGER = logging.getLogger(__name__)


async def async_setup_platform(
        hass, config, async_add_devices, discovery_info=None):
    """Set up the RainMachine Switch platform."""
    if discovery_info is None:
        return

    rainmachine = hass.data[DATA_RAINMACHINE]

    binary_sensors = []
    for sensor_type in discovery_info[CONF_MONITORED_CONDITIONS]:
        name, icon = BINARY_SENSORS[sensor_type]
        binary_sensors.append(
            RainMachineBinarySensor(rainmachine, sensor_type, name, icon))

    async_add_devices(binary_sensors, True)


class RainMachineBinarySensor(RainMachineEntity, BinarySensorDevice):
    """A sensor implementation for raincloud device."""

    def __init__(self, rainmachine, sensor_type, name, icon):
        """Initialize the sensor."""
        super().__init__(rainmachine)

        self._icon = icon
        self._name = name
        self._sensor_type = sensor_type
        self._state = None

    @property
    def icon(self) -> str:
        """Return the icon."""
        return self._icon

    @property
    def is_on(self):
        """Return the status of the sensor."""
        return self._state

    @property
    def should_poll(self):
        """Disable polling."""
        return False

    @property
    def unique_id(self) -> str:
        """Return a unique, HASS-friendly identifier for this entity."""
        return '{0}_{1}'.format(
            self.rainmachine.device_mac.replace(':', ''), self._sensor_type)

    @callback
    def _update_data(self):
        """Update the state."""
        self.async_schedule_update_ha_state(True)

    async def async_added_to_hass(self):
        """Register callbacks."""
        async_dispatcher_connect(
            self.hass, SENSOR_UPDATE_TOPIC, self._update_data)

    async def async_update(self):
        """Update the state."""
        if self._sensor_type == TYPE_FREEZE:
            self._state = self.rainmachine.restrictions['current']['freeze']
        elif self._sensor_type == TYPE_FREEZE_PROTECTION:
            self._state = self.rainmachine.restrictions['global'][
                'freezeProtectEnabled']
        elif self._sensor_type == TYPE_HOT_DAYS:
            self._state = self.rainmachine.restrictions['global'][
                'hotDaysExtraWatering']
        elif self._sensor_type == TYPE_HOURLY:
            self._state = self.rainmachine.restrictions['current']['hourly']
        elif self._sensor_type == TYPE_MONTH:
            self._state = self.rainmachine.restrictions['current']['month']
        elif self._sensor_type == TYPE_RAINDELAY:
            self._state = self.rainmachine.restrictions['current']['rainDelay']
        elif self._sensor_type == TYPE_RAINSENSOR:
            self._state = self.rainmachine.restrictions['current'][
                'rainSensor']
        elif self._sensor_type == TYPE_WEEKDAY:
            self._state = self.rainmachine.restrictions['current']['weekDay']



"""
Support for controlling GPIO pins of a Beaglebone Black.

For more details about this component, please refer to the documentation at
https://home-assistant.io/components/bbb_gpio/
"""
import logging

from homeassistant.const import (
    EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STOP)

REQUIREMENTS = ['Adafruit_BBIO==1.0.0']

_LOGGER = logging.getLogger(__name__)

DOMAIN = 'bbb_gpio'


def setup(hass, config):
    """Set up the BeagleBone Black GPIO component."""
    # pylint: disable=import-error
    import Adafruit_BBIO.GPIO as GPIO

    def cleanup_gpio(event):
        """Stuff to do before stopping."""
        GPIO.cleanup()

    def prepare_gpio(event):
        """Stuff to do when home assistant starts."""
        hass.bus.listen_once(EVENT_HOMEASSISTANT_STOP, cleanup_gpio)

    hass.bus.listen_once(EVENT_HOMEASSISTANT_START, prepare_gpio)
    return True


def setup_output(pin):
    """Set up a GPIO as output."""
    # pylint: disable=import-error
    import Adafruit_BBIO.GPIO as GPIO
    GPIO.setup(pin, GPIO.OUT)


def setup_input(pin, pull_mode):
    """Set up a GPIO as input."""
    # pylint: disable=import-error
    import Adafruit_BBIO.GPIO as GPIO
    GPIO.setup(pin, GPIO.IN,
               GPIO.PUD_DOWN if pull_mode == 'DOWN'
               else GPIO.PUD_UP)


def write_output(pin, value):
    """Write a value to a GPIO."""
    # pylint: disable=import-error
    import Adafruit_BBIO.GPIO as GPIO
    GPIO.output(pin, value)


def read_input(pin):
    """Read a value from a GPIO."""
    # pylint: disable=import-error
    import Adafruit_BBIO.GPIO as GPIO
    return GPIO.input(pin) is GPIO.HIGH


def edge_detect(pin, event_callback, bounce):
    """Add detection for RISING and FALLING events."""
    # pylint: disable=import-error
    import Adafruit_BBIO.GPIO as GPIO
    GPIO.add_event_detect(
        pin, GPIO.BOTH, callback=event_callback, bouncetime=bounce)



"""
Support for watching multiple cryptocurrencies.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.sochain/
"""
import asyncio
import logging
from datetime import timedelta

import voluptuous as vol

import homeassistant.helpers.config_validation as cv
from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (CONF_NAME, ATTR_ATTRIBUTION)
from homeassistant.helpers.entity import Entity
from homeassistant.helpers.aiohttp_client import async_get_clientsession

REQUIREMENTS = ['python-sochain-api==0.0.2']

_LOGGER = logging.getLogger(__name__)

CONF_ADDRESS = 'address'
CONF_NETWORK = 'network'
CONF_ATTRIBUTION = "Data provided by chain.so"

DEFAULT_NAME = 'Crypto Balance'

SCAN_INTERVAL = timedelta(minutes=5)

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_ADDRESS): cv.string,
    vol.Required(CONF_NETWORK): cv.string,
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
})


@asyncio.coroutine
def async_setup_platform(hass, config, async_add_devices, discovery_info=None):
    """Set up the sochain sensors."""
    from pysochain import ChainSo
    address = config.get(CONF_ADDRESS)
    network = config.get(CONF_NETWORK)
    name = config.get(CONF_NAME)

    session = async_get_clientsession(hass)
    chainso = ChainSo(network, address, hass.loop, session)

    async_add_devices([SochainSensor(name, network.upper(), chainso)], True)


class SochainSensor(Entity):
    """Representation of a Sochain sensor."""

    def __init__(self, name, unit_of_measurement, chainso):
        """Initialize the sensor."""
        self._name = name
        self._unit_of_measurement = unit_of_measurement
        self.chainso = chainso

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the state of the sensor."""
        return self.chainso.data.get("confirmed_balance") \
            if self.chainso is not None else None

    @property
    def unit_of_measurement(self):
        """Return the unit of measurement this sensor expresses itself in."""
        return self._unit_of_measurement

    @property
    def device_state_attributes(self):
        """Return the state attributes of the sensor."""
        return {
            ATTR_ATTRIBUTION: CONF_ATTRIBUTION,
        }

    @asyncio.coroutine
    def async_update(self):
        """Get the latest state of the sensor."""
        yield from self.chainso.async_get_data()



"""Helper for aiohttp webclient stuff."""
import asyncio
import ssl
import sys

import aiohttp
from aiohttp.hdrs import USER_AGENT, CONTENT_TYPE
from aiohttp import web
from aiohttp.web_exceptions import HTTPGatewayTimeout, HTTPBadGateway
import async_timeout
import certifi

from homeassistant.core import callback
from homeassistant.const import EVENT_HOMEASSISTANT_CLOSE, __version__
from homeassistant.loader import bind_hass

DATA_CONNECTOR = 'aiohttp_connector'
DATA_CONNECTOR_NOTVERIFY = 'aiohttp_connector_notverify'
DATA_CLIENTSESSION = 'aiohttp_clientsession'
DATA_CLIENTSESSION_NOTVERIFY = 'aiohttp_clientsession_notverify'
SERVER_SOFTWARE = 'HomeAssistant/{0} aiohttp/{1} Python/{2[0]}.{2[1]}'.format(
    __version__, aiohttp.__version__, sys.version_info)


@callback
@bind_hass
def async_get_clientsession(hass, verify_ssl=True):
    """Return default aiohttp ClientSession.

    This method must be run in the event loop.
    """
    if verify_ssl:
        key = DATA_CLIENTSESSION
    else:
        key = DATA_CLIENTSESSION_NOTVERIFY

    if key not in hass.data:
        hass.data[key] = async_create_clientsession(hass, verify_ssl)

    return hass.data[key]


@callback
@bind_hass
def async_create_clientsession(hass, verify_ssl=True, auto_cleanup=True,
                               **kwargs):
    """Create a new ClientSession with kwargs, i.e. for cookies.

    If auto_cleanup is False, you need to call detach() after the session
    returned is no longer used. Default is True, the session will be
    automatically detached on homeassistant_stop.

    This method must be run in the event loop.
    """
    connector = _async_get_connector(hass, verify_ssl)

    clientsession = aiohttp.ClientSession(
        loop=hass.loop,
        connector=connector,
        headers={USER_AGENT: SERVER_SOFTWARE},
        **kwargs
    )

    if auto_cleanup:
        _async_register_clientsession_shutdown(hass, clientsession)

    return clientsession


@asyncio.coroutine
@bind_hass
def async_aiohttp_proxy_web(hass, request, web_coro, buffer_size=102400,
                            timeout=10):
    """Stream websession request to aiohttp web response."""
    try:
        with async_timeout.timeout(timeout, loop=hass.loop):
            req = yield from web_coro

    except asyncio.CancelledError:
        # The user cancelled the request
        return

    except asyncio.TimeoutError as err:
        # Timeout trying to start the web request
        raise HTTPGatewayTimeout() from err

    except aiohttp.ClientError as err:
        # Something went wrong with the connection
        raise HTTPBadGateway() from err

    try:
        yield from async_aiohttp_proxy_stream(
            hass,
            request,
            req.content,
            req.headers.get(CONTENT_TYPE)
        )
    finally:
        req.close()


@bind_hass
async def async_aiohttp_proxy_stream(hass, request, stream, content_type,
                                     buffer_size=102400, timeout=10):
    """Stream a stream to aiohttp web response."""
    response = web.StreamResponse()
    response.content_type = content_type
    await response.prepare(request)

    try:
        while True:
            with async_timeout.timeout(timeout, loop=hass.loop):
                data = await stream.read(buffer_size)

            if not data:
                await response.write_eof()
                break

            await response.write(data)

    except (asyncio.TimeoutError, aiohttp.ClientError):
        # Something went wrong fetching data, close connection gracefully
        await response.write_eof()

    except asyncio.CancelledError:
        # The user closed the connection
        pass


@callback
def _async_register_clientsession_shutdown(hass, clientsession):
    """Register ClientSession close on Home Assistant shutdown.

    This method must be run in the event loop.
    """
    @callback
    def _async_close_websession(event):
        """Close websession."""
        clientsession.detach()

    hass.bus.async_listen_once(
        EVENT_HOMEASSISTANT_CLOSE, _async_close_websession)


@callback
def _async_get_connector(hass, verify_ssl=True):
    """Return the connector pool for aiohttp.

    This method must be run in the event loop.
    """
    key = DATA_CONNECTOR if verify_ssl else DATA_CONNECTOR_NOTVERIFY

    if key in hass.data:
        return hass.data[key]

    if verify_ssl:
        ssl_context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
        ssl_context.load_verify_locations(cafile=certifi.where(),
                                          capath=None)
    else:
        ssl_context = False

    connector = aiohttp.TCPConnector(loop=hass.loop, ssl=ssl_context)
    hass.data[key] = connector

    @callback
    def _async_close_connector(event):
        """Close connector pool."""
        connector.close()

    hass.bus.async_listen_once(
        EVENT_HOMEASSISTANT_CLOSE, _async_close_connector)

    return connector



"""
Support for Pollen.com allergen and cold/flu sensors.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.pollen/
"""
import logging
from datetime import timedelta
from statistics import mean

import voluptuous as vol

import homeassistant.helpers.config_validation as cv
from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (
    ATTR_ATTRIBUTION, ATTR_STATE, CONF_MONITORED_CONDITIONS)
from homeassistant.helpers import aiohttp_client
from homeassistant.helpers.entity import Entity
from homeassistant.util import Throttle

REQUIREMENTS = ['pypollencom==2.1.0']
_LOGGER = logging.getLogger(__name__)

ATTR_ALLERGEN_GENUS = 'allergen_genus'
ATTR_ALLERGEN_NAME = 'allergen_name'
ATTR_ALLERGEN_TYPE = 'allergen_type'
ATTR_CITY = 'city'
ATTR_OUTLOOK = 'outlook'
ATTR_RATING = 'rating'
ATTR_SEASON = 'season'
ATTR_TREND = 'trend'
ATTR_ZIP_CODE = 'zip_code'

CONF_ZIP_CODE = 'zip_code'

DEFAULT_ATTRIBUTION = 'Data provided by IQVIA™'
DEFAULT_SCAN_INTERVAL = timedelta(minutes=30)

TYPE_ALLERGY_FORECAST = 'allergy_average_forecasted'
TYPE_ALLERGY_HISTORIC = 'allergy_average_historical'
TYPE_ALLERGY_INDEX = 'allergy_index'
TYPE_ALLERGY_OUTLOOK = 'allergy_outlook'
TYPE_ALLERGY_TODAY = 'allergy_index_today'
TYPE_ALLERGY_TOMORROW = 'allergy_index_tomorrow'
TYPE_ALLERGY_YESTERDAY = 'allergy_index_yesterday'
TYPE_DISEASE_FORECAST = 'disease_average_forecasted'

SENSORS = {
    TYPE_ALLERGY_FORECAST: (
        'Allergy Index: Forecasted Average', None, 'mdi:flower', 'index'),
    TYPE_ALLERGY_HISTORIC: (
        'Allergy Index: Historical Average', None, 'mdi:flower', 'index'),
    TYPE_ALLERGY_TODAY: (
        'Allergy Index: Today', TYPE_ALLERGY_INDEX, 'mdi:flower', 'index'),
    TYPE_ALLERGY_TOMORROW: (
        'Allergy Index: Tomorrow', TYPE_ALLERGY_INDEX, 'mdi:flower', 'index'),
    TYPE_ALLERGY_YESTERDAY: (
        'Allergy Index: Yesterday', TYPE_ALLERGY_INDEX, 'mdi:flower', 'index'),
    TYPE_DISEASE_FORECAST: (
        'Cold & Flu: Forecasted Average', None, 'mdi:snowflake', 'index')
}

RATING_MAPPING = [{
    'label': 'Low',
    'minimum': 0.0,
    'maximum': 2.4
}, {
    'label': 'Low/Medium',
    'minimum': 2.5,
    'maximum': 4.8
}, {
    'label': 'Medium',
    'minimum': 4.9,
    'maximum': 7.2
}, {
    'label': 'Medium/High',
    'minimum': 7.3,
    'maximum': 9.6
}, {
    'label': 'High',
    'minimum': 9.7,
    'maximum': 12
}]

TREND_FLAT = 'Flat'
TREND_INCREASING = 'Increasing'
TREND_SUBSIDING = 'Subsiding'

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_ZIP_CODE): str,
    vol.Required(CONF_MONITORED_CONDITIONS, default=list(SENSORS)):
        vol.All(cv.ensure_list, [vol.In(SENSORS)])
})


async def async_setup_platform(
        hass, config, async_add_devices, discovery_info=None):
    """Configure the platform and add the sensors."""
    from pypollencom import Client

    websession = aiohttp_client.async_get_clientsession(hass)

    data = PollenComData(
        Client(config[CONF_ZIP_CODE], websession),
        config[CONF_MONITORED_CONDITIONS])

    await data.async_update()

    sensors = []
    for kind in config[CONF_MONITORED_CONDITIONS]:
        name, category, icon, unit = SENSORS[kind]
        sensors.append(
            PollencomSensor(
                data, config[CONF_ZIP_CODE], kind, category, name, icon, unit))

    async_add_devices(sensors, True)


def calculate_average_rating(indices):
    """Calculate the human-friendly historical allergy average."""
    ratings = list(
        r['label'] for n in indices for r in RATING_MAPPING
        if r['minimum'] <= n <= r['maximum'])
    return max(set(ratings), key=ratings.count)


class PollencomSensor(Entity):
    """Define a Pollen.com sensor."""

    def __init__(self, pollencom, zip_code, kind, category, name, icon, unit):
        """Initialize the sensor."""
        self._attrs = {ATTR_ATTRIBUTION: DEFAULT_ATTRIBUTION}
        self._category = category
        self._icon = icon
        self._name = name
        self._state = None
        self._type = kind
        self._unit = unit
        self._zip_code = zip_code
        self.pollencom = pollencom

    @property
    def available(self):
        """Return True if entity is available."""
        return bool(
            self.pollencom.data.get(self._type)
            or self.pollencom.data.get(self._category))

    @property
    def device_state_attributes(self):
        """Return the device state attributes."""
        return self._attrs

    @property
    def icon(self):
        """Return the icon."""
        return self._icon

    @property
    def name(self):
        """Return the name."""
        return self._name

    @property
    def state(self):
        """Return the state."""
        return self._state

    @property
    def unique_id(self):
        """Return a unique, HASS-friendly identifier for this entity."""
        return '{0}_{1}'.format(self._zip_code, self._type)

    @property
    def unit_of_measurement(self):
        """Return the unit the value is expressed in."""
        return self._unit

    async def async_update(self):
        """Update the sensor."""
        await self.pollencom.async_update()
        if not self.pollencom.data:
            return

        if self._category:
            data = self.pollencom.data[self._category]['Location']
        else:
            data = self.pollencom.data[self._type]['Location']

        indices = [p['Index'] for p in data['periods']]
        average = round(mean(indices), 1)
        [rating] = [
            i['label'] for i in RATING_MAPPING
            if i['minimum'] <= average <= i['maximum']
        ]
        slope = (data['periods'][-1]['Index'] - data['periods'][-2]['Index'])
        trend = TREND_FLAT
        if slope > 0:
            trend = TREND_INCREASING
        elif slope < 0:
            trend = TREND_SUBSIDING

        if self._type == TYPE_ALLERGY_FORECAST:
            outlook = self.pollencom.data[TYPE_ALLERGY_OUTLOOK]

            self._attrs.update({
                ATTR_CITY: data['City'].title(),
                ATTR_OUTLOOK: outlook['Outlook'],
                ATTR_RATING: rating,
                ATTR_SEASON: outlook['Season'].title(),
                ATTR_STATE: data['State'],
                ATTR_TREND: outlook['Trend'].title(),
                ATTR_ZIP_CODE: data['ZIP']
            })
            self._state = average
        elif self._type == TYPE_ALLERGY_HISTORIC:
            self._attrs.update({
                ATTR_CITY: data['City'].title(),
                ATTR_RATING: calculate_average_rating(indices),
                ATTR_STATE: data['State'],
                ATTR_TREND: trend,
                ATTR_ZIP_CODE: data['ZIP']
            })
            self._state = average
        elif self._type in (TYPE_ALLERGY_TODAY, TYPE_ALLERGY_TOMORROW,
                            TYPE_ALLERGY_YESTERDAY):
            key = self._type.split('_')[-1].title()
            [period] = [p for p in data['periods'] if p['Type'] == key]
            [rating] = [
                i['label'] for i in RATING_MAPPING
                if i['minimum'] <= period['Index'] <= i['maximum']
            ]

            for idx, attrs in enumerate(period['Triggers']):
                index = idx + 1
                self._attrs.update({
                    '{0}_{1}'.format(ATTR_ALLERGEN_GENUS, index):
                        attrs['Genus'],
                    '{0}_{1}'.format(ATTR_ALLERGEN_NAME, index):
                        attrs['Name'],
                    '{0}_{1}'.format(ATTR_ALLERGEN_TYPE, index):
                        attrs['PlantType'],
                })

            self._attrs.update({
                ATTR_CITY: data['City'].title(),
                ATTR_RATING: rating,
                ATTR_STATE: data['State'],
                ATTR_ZIP_CODE: data['ZIP']
            })
            self._state = period['Index']
        elif self._type == TYPE_DISEASE_FORECAST:
            self._attrs.update({
                ATTR_CITY: data['City'].title(),
                ATTR_RATING: rating,
                ATTR_STATE: data['State'],
                ATTR_TREND: trend,
                ATTR_ZIP_CODE: data['ZIP']
            })
            self._state = average


class PollenComData(object):
    """Define a data object to retrieve info from Pollen.com."""

    def __init__(self, client, sensor_types):
        """Initialize."""
        self._client = client
        self._sensor_types = sensor_types
        self.data = {}

    @Throttle(DEFAULT_SCAN_INTERVAL)
    async def async_update(self):
        """Update Pollen.com data."""
        from pypollencom.errors import InvalidZipError, PollenComError

        # Pollen.com requires a bit more complicated error handling, given that
        # it sometimes has parts (but not the whole thing) go down:
        #
        # 1. If `InvalidZipError` is thrown, quit everything immediately.
        # 2. If an individual request throws any other error, try the others.

        try:
            if TYPE_ALLERGY_FORECAST in self._sensor_types:
                try:
                    data = await self._client.allergens.extended()
                    self.data[TYPE_ALLERGY_FORECAST] = data
                except PollenComError as err:
                    _LOGGER.error('Unable to get allergy forecast: %s', err)
                    self.data[TYPE_ALLERGY_FORECAST] = {}

                try:
                    data = await self._client.allergens.outlook()
                    self.data[TYPE_ALLERGY_OUTLOOK] = data
                except PollenComError as err:
                    _LOGGER.error('Unable to get allergy outlook: %s', err)
                    self.data[TYPE_ALLERGY_OUTLOOK] = {}

            if TYPE_ALLERGY_HISTORIC in self._sensor_types:
                try:
                    data = await self._client.allergens.historic()
                    self.data[TYPE_ALLERGY_HISTORIC] = data
                except PollenComError as err:
                    _LOGGER.error('Unable to get allergy history: %s', err)
                    self.data[TYPE_ALLERGY_HISTORIC] = {}

            if all(s in self._sensor_types
                   for s in [TYPE_ALLERGY_TODAY, TYPE_ALLERGY_TOMORROW,
                             TYPE_ALLERGY_YESTERDAY]):
                try:
                    data = await self._client.allergens.current()
                    self.data[TYPE_ALLERGY_INDEX] = data
                except PollenComError as err:
                    _LOGGER.error('Unable to get current allergies: %s', err)
                    self.data[TYPE_ALLERGY_TODAY] = {}

            if TYPE_DISEASE_FORECAST in self._sensor_types:
                try:
                    data = await self._client.disease.extended()
                    self.data[TYPE_DISEASE_FORECAST] = data
                except PollenComError as err:
                    _LOGGER.error('Unable to get disease forecast: %s', err)
                    self.data[TYPE_DISEASE_FORECAST] = {}

            _LOGGER.debug('New data retrieved: %s', self.data)
        except InvalidZipError:
            _LOGGER.error(
                'Cannot retrieve data for ZIP code: %s', self._client.zip_code)
            self.data = {}
